# UFF Project

This project is a standard format for personal financial transactions with accompanying tools for converting, validating, and categorising a financial record.

- fint-to-uff: convert transactions from [fint](https://git.is.horse/izzy/fint) to UFF
- uff-validator: toolbox for validating, normalising, and categorising financial record

Tools are built using Bun and Typescript.

## Specification

**Financial Record**

A financial record has the following folder structure:

```
financial-record
|- [account ID]
  |- [YYYY-MM]
    |- [YYYYMMDD-C].uff1.json   Type 1 UFF file *
    |- [YYYYMMDD-C].uff2.json   Type 2 UFF file (optional)
  |- pending
    |- [...].uff1/2.json        As above, except these transactions are not confirmed
  |- rules.js                   Account-speciifc rules for categorising data
|- rules.js                     Global rules for categorising data
```

<sup>\*</sup> `YYYYMMDD-C` is the year, month, day, and counter (of transactions that day) which forms this transaction's unique ID

It should be initialised as a Git repository and placed into `./financial-record` in this workspace.

**Type 1 UFF file** (`.uff1.json`)

This file contains basic information about a transaction along with any raw data provided by the source it was imported from. It is serialised as JSON. All fields must exist.

| Field  |  Type  | Description                                                            |
| :----: | :----: | :--------------------------------------------------------------------- |
| `time` | String | Valid ISO8601 Timestamp                                                |
| `net`  | String | Net balance change (normalised as `^-?(0 \| [1-9][0-9]\*)\.[0-9]{2}$`) |
| `data` |  Any   | Any valid JSON data type                                               |

**Type 2 UFF file** (`.uff2.json`)

This file is a descriptor file, containing structured metadata generated or added to information in UFF1. It is serialised as JSON. All fields are optional. If this file does not exist, assume all fields are null.

|     Field     |        Type         | Description                                                                        |
| :-----------: | :-----------------: | :--------------------------------------------------------------------------------- |
| `description` |       String        | Freeform information about this transaction                                        |
|    `payee`    |       String        | Who this was sent/received to/from <br> Use `Self` for transfers between accounts. |
|    `type`     |  Enum <sup>†</sup>  | Type of transaction                                                                |
|  `category`   |       String        | Category of transaction                                                            |
|    `links`    | Link[] <sup>‡</sup> | List of transactions this is linked to                                             |

<sup>†</sup> Table describing TransactionType enum fields

|     Type      | Meaning                                                  |          Direction          |
| :-----------: | :------------------------------------------------------- | :-------------------------: |
|     Dummy     | Zero net transaction                                     |                             |
|   Transfer    | Money going between accounts                             |                             |
|  Conversion   | Special-case of transfer where currency is converted     |                             |
|    Income     | Money coming in considered income                        |           Income            |
|   Interest    | Interest on balance                                      |           Income            |
| Reimbursement | Money coming in considered a reimbursement for something |           Income            |
|     Loan      | Money coming in considered a reimbursement for something | Income/Expense <sup>§</sup> |
|    Expense    | Money going out                                          |           Expense           |
| Compensation  | Money going out considered compensation                  |           Expense           |

<sup>§</sup> A loan is categorised as 'income' if net is positive, otherwise 'expense'.

<sup>‡</sup> Table describing Link schema

|      Field      |       Type        | Description                               |
| :-------------: | :---------------: | :---------------------------------------- |
|     `type`      | Enum <sup>¶</sup> | Type of link to other transaction         |
|   `accountId`   |      String       | Account ID in record                      |
| `transactionId` |      String       | Transaction ID in a record                |
|    `comment`    |      String?      | An optional comment on why this is linked |

<sup>¶</sup> Value is `Direct` if we are linking to a corresponding cause/effect transaction (e.g. a transfer) otherwise it is `Indirect` if the corresponding transaction is simply 'connected' (e.g. someone paying back for something).

<p align="center">
![XKCD Standards](https://imgs.xkcd.com/comics/standards.png)
</p>
