export type FintAccount = {
  _id: string;
  name: string;
  currency: string;
  hashingStrategy: "txid" | "paypal" | "revolut" | "natwest";
  uncontrolled: boolean;
};

/**
 * Transaction schema from GoCardless
 * This is determined by Open Banking standards
 */
export type GoCardlessTransactionSchema = {
  transactionAmount: {
    amount: string;
    currency: string;
  };
} & Partial<{
  transactionId: string;
  internalTransactionId: string;

  entryReference: string;
  endToEndId: string;
  mandateId: string;
  checkId: string;
  creditorId: string;
  bookingDate: string;
  valueDate: string;
  bookingDateTime: string;
  valueDateTime: string;
  currencyExchange: Partial<{
    sourceCurrency: string;
    exchangeRate: string;
    unitCurrency: string;
    targetCurrency: string;
    quotationDate: string;
    contractIdentification: string;
  }>;
  creditorName: string;
  creditorAccount: Partial<{
    iban: string;
    bban: string;
    pan: string;
    maskedPan: string;
    msisdn: string;
    currency: string;
  }>;
  ultimateCreditor: string;
  debtorName: string;
  debtorAccount: Partial<{
    iban: string;
    bban: string;
    pan: string;
    maskedPan: string;
    msisdn: string;
    currency: string;
  }>;
  ultimateDebtor: string;
  remittanceInformationUnstructured: string;
  remittanceInformationUnstructuredArray: string[];
  remittanceInformationStructured: string;
  remittanceInformationStructuredArray: string[];
  additionalInformation: string;
  purposeCode: string;
  bankTransactionCode: string;
  proprietaryBankTransactionCode: string;
}>;

export type RawTransactionRevolut = {
  Type: string;
  Product: string;
  "Started Date": string;
  "Completed Date": string;
  Description: string;
  Amount: string;
  Fee: string;
  Currency: string;
  State: string;
  Balance: string;
};

export type RawTransactionPayPal = {
  Date: string;
  Time: string;
  "Time zone": string;
  Name: string;
  Type: string;
  Status: string;
  Currency: string;
  Gross: string;
  Fee: string;
  Net: string;
  "From Email Address": string;
  "To Email Address": string;
  "Transaction ID": string;
  "Reference Txn ID": string;
  Balance: string;
  Subject: string;
  Note: string;
  "Balance Impact": "Credit" | "Debit" | "Memo";
};

export type RawTransactionNatWest = {
  Date: string;
  Description: string;
  Balance: string;
  Value: string;
  Type: string;
  "Account Name": string;
  "Account Number": string;
};

export type FintRawTransaction = {
  hash: string;
  pending: boolean;
} & (
  | {
      _id: {
        accountId: string;
        source: "gocardless";
        transactionId: string;
      };

      data: GoCardlessTransactionSchema;
    }
  | {
      _id: {
        accountId: string;
        source: "manual_natwest";
        transactionId: string;
      };

      data: RawTransactionNatWest;
    }
  | {
      _id: {
        accountId: string;
        source: "manual_revolut";
        transactionId: string;
      };

      data: RawTransactionRevolut;
    }
  | {
      _id: {
        accountId: string;
        source: "manual_paypal";
        transactionId: string;
      };

      data: RawTransactionPayPal;
    }
);
