import { inspect } from "bun";
import {
  FintRawTransaction,
  GoCardlessTransactionSchema,
  RawTransactionNatWest,
  RawTransactionPayPal,
  RawTransactionRevolut,
} from "../db/types.fint";

export function getAmount(rawEntries: FintRawTransaction[]): string {
  // PayPal
  const paypalEntries = rawEntries.filter(
    (entry) => entry._id.source === "manual_paypal"
  ) as { data: RawTransactionPayPal }[];

  if (paypalEntries.length) {
    return (
      paypalEntries.reduce(
        (c, entry) => c + parseInt(entry.data.Net.replace(/,|\./g, "")),
        0
      ) / 100
    ).toString();
  }

  // NatWest
  const natwestEntries = rawEntries.filter(
    (entry) => entry._id.source === "manual_natwest"
  ) as { data: RawTransactionNatWest }[];

  if (natwestEntries.length) {
    return natwestEntries[0].data.Value.replace(/,/g, "");
  }

  // GoCardless
  const goCardlessEntries = rawEntries.filter(
    (entry) => entry._id.source === "gocardless"
  ) as { data: GoCardlessTransactionSchema }[];
  if (goCardlessEntries.length) {
    return goCardlessEntries[0].data.transactionAmount.amount;
  }

  return "0.00";
}

export function getDate(rawEntries: FintRawTransaction[]): Date {
  for (const entry of rawEntries) {
    switch (entry._id.source) {
      case "gocardless":
        return new Date(
          (entry.data as GoCardlessTransactionSchema).bookingDate!
        );
      case "manual_natwest": {
        const D = (entry.data as RawTransactionNatWest).Date;
        const [dd, mm, yyyy] = D.split("/");
        return new Date(`${yyyy}-${mm}-${dd}`);
      }
      case "manual_paypal": {
        const [dd, mm, yyyy] = (entry.data as RawTransactionPayPal).Date.split(
          "/"
        );

        const [HH, MM, ss] = (entry.data as RawTransactionPayPal).Time.split(
          ":"
        );

        let timezone;
        const tz = (entry.data as RawTransactionPayPal)["Time zone"];
        switch (tz) {
          case "UTC":
          case "GMT":
            timezone = "Z";
            break;
          case "BST":
            timezone = "+01:00";
            break;
          default:
            throw Error("erm... what the sigma? " + tz);
        }

        return new Date(`${yyyy}-${mm}-${dd}T${HH}:${MM}:${ss}.000${timezone}`);
      }
      case "manual_revolut":
        return new Date(
          (entry.data as RawTransactionRevolut)["Completed Date"]
        );
    }
  }

  return new Date("invalid");
}
