"use client";

import dayjs from "dayjs";
import {
  Area,
  AreaChart,
  CartesianGrid,
  ResponsiveContainer,
  Tooltip,
  XAxis,
  YAxis,
} from "recharts";

export function ChartIt({
  data,
  yDomain,
}: {
  data: { time: number; balance: number | null; projected: number | null }[];
  yDomain?: [number, number];
}) {
  // const sd = new Date("2021-01-01");
  const sd = new Date("2024-05-01");

  // const ed = new Date("2024-12-31");
  const ed = new Date("2025-06-01");

  return (
    <ResponsiveContainer width="100%" height={500}>
      <AreaChart data={data.filter((x) => x.time >= +sd)}>
        <CartesianGrid strokeDasharray="3 3" />

        <Tooltip />

        <XAxis
          name="Date"
          type="number"
          dataKey="time"
          // domain={[data[0].time, data[data.length - 1].time]}
          domain={[+sd, +ed]}
          tickFormatter={(unixTime) => dayjs(unixTime).format("DD MMM YYYY")}
        />

        <YAxis
          name="Balance"
          type="number"
          dataKey="balance"
          domain={yDomain}
          tickFormatter={(balance) => "£" + balance.toLocaleString()}
        />

        <Area
          type="monotone"
          dataKey="revolt"
          stroke="#ff000088"
          fill="transparent"
        />

        <Area
          type="monotone"
          dataKey="balance"
          stroke="#8884d888"
          fill="#8884d822"
        />

        <Area
          type="monotone"
          dataKey="projected"
          stroke="#8884d888"
          fill="transparent"
        />
      </AreaChart>
    </ResponsiveContainer>
  );
}
