import { FinancialRecord } from "../../uff-validator/lib/uff";
import { TransactionUtil } from "../../uff-validator/lib/util";
import { ChartIt } from "./Chart";

/*import {
  Area,
  AreaChart,
  CartesianGrid,
  Legend,
  LineChart,
  ResponsiveContainer,
  Scatter,
  ScatterChart,
  Tooltip,
  XAxis,
  YAxis,
} from "recharts";

import theData from "./d.json";
import dayjs from "dayjs";

const end = new Date();
const start = new Date(`${end.getUTCFullYear()}-01-01`);
// const start = new Date(`${end.getUTCFullYear()}-${end.getUTCMonth() - 1}-01`);
const startT = +start;
const endT = +end;

const dataF = theData
  .filter(({ x }) => x >= startT && x <= endT)
  .map(({ x, y }) => ({
    time: x,
    balance: y,
  }));

dataF.unshift({
  time: startT,
  balance: dataF[0].balance,
});

dataF.push({
  time: endT,
  balance: dataF[dataF.length - 1].balance
});*/

export default async function Home() {
  const record = await FinancialRecord.fromDirectory("../financial-record");

  function gen(util: TransactionUtil, util2?: TransactionUtil) {
    const entries = [],
      currentDate = util.first.time,
      endDate = util.last.time,
      resolution = 1;

    currentDate.setHours(23);
    currentDate.setMinutes(59);
    currentDate.setSeconds(58);

    endDate.setHours(23);
    endDate.setMinutes(59);
    endDate.setSeconds(59);

    while (currentDate <= endDate) {
      entries.push({
        time: +new Date(currentDate.toISOString().substring(0, 10)),
        balance: util.satisfy((t) => t.time <= currentDate).net,
        revolt: util2 ? util2.satisfy((t) => t.time <= currentDate).net : null,
        projected: null,
      });

      currentDate.setDate(currentDate.getDate() + resolution);
    }

    let sum = util.net;
    const roltSum = util2 ? util2.net : null;
    entries[entries.length - 1].projected = sum;

    const netAffecting = {
      // smarty
      "2024-08-29": -12,
      "2024-09-29": -12,
      "2024-10-29": -12,
      "2024-11-29": -12,
      "2024-12-29": -12,
      "2025-01-29": -12,
      "2025-02-29": -12,
      "2025-03-29": -12,
      "2025-04-29": -12,
      "2025-05-29": -12,
      "2025-06-29": -12,
      "2025-07-29": -12,
      // slc
      "2024-09-16": 1149.06,
      "2025-01-06": 1149.06,
      "2025-04-22": 1183.88,
      // current tfl forecasting
      "2024-09-23": -30, // 3 days of travel
      "2024-09-30": -30, // 3 days of travel
      "2024-10-07": -30, // 3 days of travel
      "2024-10-14": -30, // 3 days of travel
      "2024-10-21": -30, // 3 days of travel
      "2024-11-04": -30, // 3 days of travel
      "2024-11-11": -40, // 4 days of travel
      "2024-11-18": -40, // 4 days of travel
      "2024-12-02": -40, // 4 days of travel
      "2024-12-09": -10, // 1 days of travel
    };

    const balanceAffecting = {
      ...netAffecting,
      // move to savings account
      "2024-08-24": -150,
      "2024-09-24": -150,
      "2024-10-24": -150,
      "2024-11-24": -150,
      "2024-12-24": -150,
      "2025-01-24": -150,
      "2025-02-24": -150,
      "2025-03-24": -150,
      "2025-04-24": -150,
      "2025-05-24": -150,
      "2025-06-24": -150,
      "2025-07-24": -150,
    };

    const projectEnd = new Date("2025-06-01");
    while (currentDate < projectEnd) {
      const ymd = currentDate.toISOString().substring(0, 10);

      const src = roltSum ? netAffecting : balanceAffecting; // if on net worth
      if (src[ymd]) sum += src[ymd];

      entries.push({
        time: +new Date(ymd),
        balance: null,
        revolt: roltSum,
        projected: sum,
      });

      currentDate.setDate(currentDate.getDate() + resolution);
    }

    return entries;
  }

  const utilCurrentAccount = new TransactionUtil(
    record.getAccount(
      record.accountIds.find((x) => x.endsWith("current_account"))!
    ).allTransactions
  );

  const utilCombinedNetWorth = new TransactionUtil(
    record.accountIds
      .filter(
        (x) =>
          x.endsWith("current_account") ||
          x.endsWith("saver") ||
          x.endsWith("_gbp")
      )
      .flatMap((x) => record.getAccount(x).allTransactions)
  );

  const utilPersonalNetWorth = new TransactionUtil(
    record.accountIds
      .filter(
        (x) =>
          x.endsWith("current_account") ||
          x.endsWith("saver") ||
          x.endsWith("_gbp")
      )
      .flatMap((x) => record.getAccount(x).allTransactions)
  ).pool();

  const utilRevolt = new TransactionUtil(
    record.accountIds
      .filter(
        (x) =>
          x.endsWith("current_account") ||
          x.endsWith("saver") ||
          x.endsWith("_gbp")
      )
      .flatMap((x) => record.getAccount(x).allTransactions)
  ).pool("Revolt");

  return (
    <main className="flex flex-col p-4 gap-4">
      <h1 className="text-xl">Current Account</h1>
      <ChartIt data={gen(utilCurrentAccount)} />

      <h1 className="text-xl">Combined Net Worth</h1>
      <ChartIt
        data={gen(utilCombinedNetWorth, utilRevolt)}
        yDomain={[0, 7000]}
      />

      <h1 className="text-xl">Personal Net Worth</h1>
      <ChartIt
        data={gen(utilPersonalNetWorth, utilRevolt)}
        yDomain={[-1400, 6000]}
      />

      {/*<ResponsiveContainer width="100%" height={500}>
        <AreaChart data={dataF}>
          <XAxis
            name="Date"
            type="number"
            dataKey="time"
            domain={[startT, endT]}
            tickFormatter={(unixTime) => dayjs(unixTime).format("DD MMM YYYY")}
          />
          <YAxis
            name="Balance"
            type="number"
            dataKey="balance"
            tickFormatter={(balance) => "£" + balance.toLocaleString()}
          />

          <Area
            type="monotone"
            dataKey="balance"
            stroke="#8884d888"
            fill="#8884d822"
          />
        </AreaChart>
      </ResponsiveContainer>*/}
    </main>
  );
}
