import { FinancialRecord } from "./lib/uff";
import { writeFile, readFile } from "fs/promises";

const record = await FinancialRecord.fromDirectory("../financial-record");

for (const id of record.accountIds) {
  const account = record.getAccount(id);
  for (const transaction of account.allTransactions) {
    const normalised = parseFloat(transaction.net).toFixed(2);
    if (normalised !== transaction.net) {
      console.info(
        "Correcting",
        transaction.net,
        "to",
        normalised,
        "for",
        transaction.id,
        `(${id})`
      );

      const { time, data } = JSON.parse(
        await readFile(transaction.filePath).then((f) => f.toString())
      );

      await writeFile(
        transaction.filePath,
        JSON.stringify(
          {
            time,
            net: normalised,
            data,
          },
          null,
          2
        )
      );
    }
  }
}
